Class: Tabula::Render::PageDrawerNoText

Inherits:
PageDrawer
  • Object
show all
Defined in:
lib/tabula/pdf_render.rb

Overview

render a PDF page to a graphics context, but skip rendering the text This is done to reduce ‘noise’ introduced by the text, we only care about lines.

Instance Method Summary collapse

Instance Method Details

#processTextPosition(text) ⇒ Object



17
18
# File 'lib/tabula/pdf_render.rb', line 17

def processTextPosition(text)
end